@@ -30,8 +30,9 @@ var NAVTREEINDEX =
 "gphoto2-camera_8c.html",
 "gphoto2-file_8h.html#aca2b7ed7d1b91fcdb0578ee71142a101",
 "gphoto2-port-info-list_8h.html#af1f1553e664584f74724a8825829bc09",
-"gphoto2-version_8h.html",
-"struct__CameraFilesystem.html#a4f95116fa657186a1cb011004451d503"
+"gphoto2-version_8c.html#a039629a84ef4c735b1124901438a7468",
+"struct__CameraFilesystem.html#a48290da7bdd03d2a9f24a4bf6fa840dd",
+"union__GPPortSettings.html#ac5d97643d4d2c83edd17731949966b49"
 ];
 
 var SYNCONMSG = 'click to disable panel synchronisation';
