@@ -1,5 +1,6 @@
 var NAVTREEINDEX5 =
 {
+"struct__CameraFilesystem.html#a48290da7bdd03d2a9f24a4bf6fa840dd":[1,0,8,12],
 "struct__CameraFilesystem.html#a4f95116fa657186a1cb011004451d503":[1,0,8,6],
 "struct__CameraFilesystem.html#a6a7b0038371ba041f35153ad55e302ba":[1,0,8,16],
 "struct__CameraFilesystem.html#a93e23f96c30c239d9db008156eb9ecf1":[1,0,8,9],
@@ -248,6 +249,5 @@ var NAVTREEINDEX5 =
 "union__GPPortSettings.html":[1,0,24],
 "union__GPPortSettings.html#a0aefcc2253c254cac9ea3fb15f2a059b":[1,0,24,1],
 "union__GPPortSettings.html#a5e89705555ef3dd1abc7baca312cd253":[1,0,24,3],
-"union__GPPortSettings.html#a9fa1454110af3b6425425de105284c8e":[1,0,24,2],
-"union__GPPortSettings.html#ac5d97643d4d2c83edd17731949966b49":[1,0,24,0]
+"union__GPPortSettings.html#a9fa1454110af3b6425425de105284c8e":[1,0,24,2]
 };
